@@ -34,30 +34,31 @@ class AssetBundleCreator(AssetBundleCreatorBase):
     For more information, see: `Documentation/misc_frontend/add_local_object.md`.
     """
 
-    def __init__(self, quiet: bool = False, display: str = ":0"):
+    def __init__(self, quiet: bool = False, display: str = ":0", unity_editor_path: Union[Path, str] = None):
         """
         :param quiet: If true, don't print any messages to console.
         :param display: The display to launch Unity Editor on. Ignored if this isn't Linux.
+        :param unity_editor_path: The path to the Unity Editor executable, for example `C:/Program Files/Unity/Hub/Editor/2020.3.24f1/Editor/Unity.exe`. If None, this script will try to find Unity Editor automatically.
         """
 
-        super().__init__(quiet=quiet, display=display)
+        super().__init__(quiet=quiet, display=display, unity_editor_path=unity_editor_path)
 
         system = platform.system()
-        self.binaries: Dict[str, str] = dict()
+        self._binaries: Dict[str, str] = dict()
         binary_path = f"binaries/{system}"
 
         # Cache the binaries.
-        self.binaries["assimp"] = f"{binary_path}/assimp/assimp"
-        self.binaries["meshconv"] = f"{binary_path}/meshconv/meshconv"
-        self.binaries["vhacd"] = f"{binary_path}/vhacd/testVHACD"
+        self._binaries["assimp"] = f"{binary_path}/assimp/assimp"
+        self._binaries["meshconv"] = f"{binary_path}/meshconv/meshconv"
+        self._binaries["vhacd"] = f"{binary_path}/vhacd/testVHACD"
 
-        for binary in self.binaries:
+        for binary in self._binaries:
             # Add the .exe suffix for Windows.
             if system == "Windows":
-                self.binaries[binary] += ".exe"
+                self._binaries[binary] += ".exe"
             # Run chmod +x on everything.
             else:
-                call(["chmod", "+x", pkg_resources.resource_filename(__name__, self.binaries[binary])])
+                call(["chmod", "+x", pkg_resources.resource_filename(__name__, self._binaries[binary])])
 
     def create_asset_bundle(self, model_path: Union[Path, str], cleanup: bool, wnid: int = -1, wcategory: str = "", scale: float = 1) -> (List[Path], Path):
         """
@@ -110,10 +111,10 @@ class AssetBundleCreator(AssetBundleCreatorBase):
             if materials_directory.exists():
                 shutil.rmtree(str(materials_directory.resolve()))
 
-            if not self.quiet:
+            if not self._quiet:
                 print("Removed temporary files.")
 
-        if not self.quiet:
+        if not self._quiet:
             print("DONE!")
 
         return asset_bundle_paths, record_path
@@ -141,7 +142,7 @@ class AssetBundleCreator(AssetBundleCreatorBase):
         :return The path to `<home>/asset_bundle_creator/Assets/`
         """
 
-        assets_directory = self.project_path.joinpath("Assets")
+        assets_directory = self._project_path.joinpath("Assets")
         assert assets_directory.exists(), f"Assets directory not found: {assets_directory.resolve()}"
 
         return assets_directory
@@ -184,16 +185,16 @@ class AssetBundleCreator(AssetBundleCreatorBase):
         if unity_project_path.exists():
             return unity_project_path
 
-        if not self.quiet:
+        if not self._quiet:
             print(f"Creating: {unity_project_path.resolve()}")
 
         call([str(AssetBundleCreatorBase.get_editor_path().resolve()),
               "-createProject",
               str(unity_project_path.resolve()),
               "-quit",
-              "-batchmode"], env=self.env)
+              "-batchmode"], env=self._env)
         assert unity_project_path.exists(), unity_project_path.resolve()
-        if not self.quiet:
+        if not self._quiet:
             print(f"Created new Unity project: {str(unity_project_path.resolve())}")
         # Add the .unitypackage to the new project.
         package_name = "asset_bundle_creator.unitypackage"
@@ -206,8 +207,8 @@ class AssetBundleCreator(AssetBundleCreatorBase):
               "-importPackage",
               filepath,
               "-quit",
-              "-batchmode"], env=self.env)
-        if not self.quiet:
+              "-batchmode"], env=self._env)
+        if not self._quiet:
             print(f"Imported {package_name} into the new project.")
 
     @staticmethod
@@ -229,17 +230,17 @@ class AssetBundleCreator(AssetBundleCreatorBase):
 
         if model_path.suffix != ".fbx":
             assert model_path.suffix == ".obj"
-            if not self.quiet:
+            if not self._quiet:
                 print("Model is already a .obj file. Skipping the conversion to .obj")
             return model_path, False
 
-        if not self.quiet:
+        if not self._quiet:
             print("Converting a .fbx file to .obj")
 
         # Create the .obj file.
         obj_filename = model_path.stem + ".obj"
 
-        assimp = pkg_resources.resource_filename(__name__, self.binaries["assimp"])
+        assimp = pkg_resources.resource_filename(__name__, self._binaries["assimp"])
         assert Path(assimp).exists(), assimp
 
         # Run assimp to create the .obj file.
@@ -255,7 +256,7 @@ class AssetBundleCreator(AssetBundleCreatorBase):
         mtl_path = Path(obj_filename + ".mtl")
         if mtl_path.exists():
             mtl_path.unlink()
-            if not self.quiet:
+            if not self._quiet:
                 print("Removed superfluous .obj.mtl file")
 
         return obj_path, True
@@ -278,11 +279,11 @@ class AssetBundleCreator(AssetBundleCreatorBase):
         assert Path(obj_path).exists(), f"Target .obj doesn't exist: {obj_path}"
 
         wrl_filename = model_path.stem + ".wrl"
-        if not self.quiet:
+        if not self._quiet:
             print("Running V-HACD on a .obj file (this might take awhile).")
 
         # Run V-HACD.
-        vhacd = pkg_resources.resource_filename(__name__, self.binaries["vhacd"])
+        vhacd = pkg_resources.resource_filename(__name__, self._binaries["vhacd"])
 
         assert Path(vhacd).exists(), vhacd
         call([vhacd,
@@ -295,21 +296,21 @@ class AssetBundleCreator(AssetBundleCreatorBase):
                                             "Check your original file to make sure that is 3D " \
                                             "(not just a flat plane) and isn't corrupted."
 
-        if not self.quiet:
+        if not self._quiet:
             print(f"Created: {wrl_filename}")
 
         # Remove the log, if any.
         if Path("log.txt").exists():
             os.remove("log.txt")
 
-            if not self.quiet:
+            if not self._quiet:
                 print("Removed V-HACD log file.")
 
         # Remove the superfluous .obj file.
         if model_path.suffix != ".obj":
             os.remove(obj_path)
 
-            if not self.quiet:
+            if not self._quiet:
                 print(f"Removed superfluous .obj file: {obj_path}")
 
         return Path(wrl_filename)
@@ -326,11 +327,11 @@ class AssetBundleCreator(AssetBundleCreatorBase):
 
         assert wrl_filename.exists(), f"Missing .wrl file: {wrl_filename}"
 
-        if not self.quiet:
+        if not self._quiet:
             print("Converting .wrl to .obj")
 
         # Run meshconv.
-        meshconv = pkg_resources.resource_filename(__name__, self.binaries["meshconv"])
+        meshconv = pkg_resources.resource_filename(__name__, self._binaries["meshconv"])
         assert Path(meshconv).exists(), meshconv
         call([meshconv,
               str(wrl_filename.resolve()),
@@ -344,7 +345,7 @@ class AssetBundleCreator(AssetBundleCreatorBase):
         obj_path = Path(f"{model_name}_colliders.obj")
         assert obj_path.exists(), "Failed to create .obj file from .wrl file."
 
-        if not self.quiet:
+        if not self._quiet:
             print(f"Created {obj_path.resolve()} from .wrl file.")
 
         return obj_path
@@ -365,7 +366,7 @@ class AssetBundleCreator(AssetBundleCreatorBase):
 
         resources_directory = self.get_resources_directory()
 
-        if not self.quiet:
+        if not self._quiet:
             print("Copying files into the Unity project.")
 
         mtl = model_path.parent.joinpath(Path(model_path.stem + ".mtl"))
@@ -398,7 +399,7 @@ class AssetBundleCreator(AssetBundleCreatorBase):
             else:
                 distutils.dir_util.copy_tree(str(src.resolve()), str(dest.resolve()))
             dests.append(dest)
-        if not self.quiet:
+        if not self._quiet:
             print("Copied files into the Unity project.")
         return dests
 
@@ -416,13 +417,13 @@ class AssetBundleCreator(AssetBundleCreatorBase):
         report = self.get_assets_directory().joinpath("report.txt")
         if report.exists():
             os.remove(str(report.resolve()))
-            if not self.quiet:
+            if not self._quiet:
                 print("Removed old report.")
 
         # Create the prefab.
-        if not self.quiet:
+        if not self._quiet:
             print("Creating the prefab...")
-        prefab_call = self.unity_call[:]
+        prefab_call = self._unity_call[:]
         prefab_call.extend(["-executeMethod",
                             "AssetBundleCreator.CreatePrefab",
                             "-filename=" + model_name,
@@ -430,7 +431,7 @@ class AssetBundleCreator(AssetBundleCreatorBase):
                             "-extension=" + model_extension,
                             "-colliders=" + colliders
                             ])
-        call(prefab_call, env=self.env)
+        call(prefab_call, env=self._env)
         prefab_path = self.get_resources_directory().joinpath(f"prefab/{model_name}.prefab")
         assert prefab_path.exists(), "Failed to create prefab."
 
@@ -438,7 +439,7 @@ class AssetBundleCreator(AssetBundleCreatorBase):
         if report.exists():
             raise Exception(f"Created the prefab with errors: {report.read_text()}")
 
-        if not self.quiet:
+        if not self._quiet:
             print("Created the prefab.")
 
         return prefab_path, report
@@ -459,9 +460,9 @@ class AssetBundleCreator(AssetBundleCreatorBase):
 
         assert prefab_path.exists(), f"Missing prefab: {prefab_path.resolve()}"
 
-        if not self.quiet:
+        if not self._quiet:
             print("Creating local asset bundles")
-        asset_bundle_call = self.unity_call[:]
+        asset_bundle_call = self._unity_call[:]
         platforms_call = ""
         for p in platforms:
             platforms_call += p + ","
@@ -471,7 +472,7 @@ class AssetBundleCreator(AssetBundleCreatorBase):
                                   "-modelname=" + model_name,
                                   "-platforms=" + platforms_call
                                   ])
-        call(asset_bundle_call, env=self.env)
+        call(asset_bundle_call, env=self._env)
         new_asset_bundles_directory = self.get_assets_directory().joinpath("NewAssetBundles")
         new_asset_bundles_directory = new_asset_bundles_directory.joinpath(model_name)
         assert new_asset_bundles_directory.exists(), f"No asset bundles found: {new_asset_bundles_directory.resolve()}"
@@ -485,7 +486,7 @@ class AssetBundleCreator(AssetBundleCreatorBase):
             assert bundle_path.exists(), f"Missing asset bundle: {asset_bundle_platform}"
             paths.append(bundle_path)
 
-        if not self.quiet:
+        if not self._quiet:
             print("Created local asset bundles.")
 
         return paths
@@ -517,7 +518,7 @@ class AssetBundleCreator(AssetBundleCreatorBase):
         """
 
         # Write the record.
-        if not self.quiet:
+        if not self._quiet:
             print("Creating a record.")
 
         if record is None:
@@ -557,12 +558,12 @@ class AssetBundleCreator(AssetBundleCreatorBase):
         record_path = self.get_assets_directory().joinpath(model_name + ".json")
         record_path.write_text(record_data, encoding="utf-8")
 
-        record_call = self.unity_call[:]
+        record_call = self._unity_call[:]
         record_call.extend(["-executeMethod",
                             "RecordCreator.WriteRecord",
                             "-modelname=" + model_name,
                             "-scale=" + str(scale)])
-        call(record_call, env=self.env)
+        call(record_call, env=self._env)
 
         # Test the record.
         try:
@@ -570,7 +571,7 @@ class AssetBundleCreator(AssetBundleCreatorBase):
         except json.JSONDecodeError:
             raise Exception("Failed to deserialize: " + record_path.read_text(encoding="utf-8"))
 
-        if not self.quiet:
+        if not self._quiet:
             print("Wrote the record data to the disk.")
 
         if write_physics:
@@ -635,7 +636,7 @@ class AssetBundleCreator(AssetBundleCreatorBase):
         :return True if there aren't problems, and a string output report.
         """
 
-        if not self.quiet:
+        if not self._quiet:
             print("Validating asset bundle...")
 
         c = Controller()
@@ -654,7 +655,7 @@ class AssetBundleCreator(AssetBundleCreatorBase):
             for problem in v.reports:
                 output += "\n\t" + problem
             return False, output
-        if not self.quiet:
+        if not self._quiet:
             print("OK!")
         return True, ""
 
@@ -692,7 +693,7 @@ class AssetBundleCreator(AssetBundleCreatorBase):
                 wrl_path.unlink()
 
         # Create the asset bundles.
-        record_call = self.unity_call[:]
+        record_call = self._unity_call[:]
         record_call.extend(["-executeMethod",
                             "AssetBundleCreator.CreateManyAssetBundles",
                             "-library=" + library_path,
@@ -700,18 +701,26 @@ class AssetBundleCreator(AssetBundleCreatorBase):
         call(record_call)
 
         if cleanup:
-            root_dir = self.get_assets_directory()
-            # Remove assets used to create asset bundles.
-            for ext in [".obj", ".fbx", ".mtl", ".mat", ".jpg", ".prefab"]:
-                for f in root_dir.rglob(f"*{ext}"):
-                    f.unlink()
-            # Remove asset bundle junk.
-            distutils.dir_util.remove_tree(str(self.get_assets_directory().joinpath("NewAssetBundles").resolve()))
-            # Remove models junk.
-            for d in models_dir.iterdir():
-                if "placeholder" in d.stem:
-                    continue
-                if d.is_file():
-                    d.unlink()
-                else:
-                    distutils.dir_util.remove_tree(str(d.resolve()))
+            self.cleanup()
+
+    def cleanup(self) -> None:
+        """
+        Delete all files from `~/asset_bundle_creator` with these extensions: .obj, .fbx, .mtl, .mat, .jpg, .prefab
+        """
+
+        root_dir = self.get_assets_directory()
+        # Remove assets used to create asset bundles.
+        for ext in [".obj", ".fbx", ".mtl", ".mat", ".jpg", ".prefab"]:
+            for f in root_dir.rglob(f"*{ext}"):
+                f.unlink()
+        # Remove asset bundle junk.
+        distutils.dir_util.remove_tree(str(self.get_assets_directory().joinpath("NewAssetBundles").resolve()))
+        # Remove models junk.
+        models_dir = self.get_resources_directory().joinpath("models")
+        for d in models_dir.iterdir():
+            if "placeholder" in d.stem:
+                continue
+            if d.is_file():
+                d.unlink()
+            else:
+                distutils.dir_util.remove_tree(str(d.resolve()))
